--import knihoven
LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

--deklarace interface bloku
ENTITY led_blikac IS 
  PORT (
    res  : IN  std_logic; --reset - vstupni signal
    clk  : IN  std_logic; --hodiny - vstupni signal
    frq  : IN  std_logic_vector (1 DOWNTO 0); --rizeni frekvence

    led  : OUT std_logic  --vystup pro ovladani diody
  );
END ENTITY led_blikac; 

--vlastni implementace bloku
ARCHITECTURE rtl OF led_blikac IS
  --interni signaly
  SIGNAL cnt_d : unsigned (21 DOWNTO 0); --citac, vstup registru
  SIGNAL cnt_q : unsigned (21 DOWNTO 0); --citac, vystup registru
  SIGNAL led_cnt_en_i  : std_logic;
  SIGNAL led_cnt_d     : unsigned ( 3 DOWNTO 0);
  SIGNAL led_cnt_q     : unsigned ( 3 DOWNTO 0);
BEGIN
  --popis registru obvodu, reaguje na hodiny a reset
  registry_citacu : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN --je aktivni asynchronni reset?
      cnt_q     <= (OTHERS => '0');
      led_cnt_q <= (OTHERS => '0');
    ELSIF clk = '1' AND clk'EVENT THEN --hodiny jsou v jednicce po zmene?
      --tedy, byla nabezna hrana na hodinach?
      cnt_q     <= cnt_d;
      led_cnt_q <= led_cnt_d;
    END IF;
  END PROCESS registry_citacu;
  --vlastni kombinacni logicka funkce
  cnt_d <= cnt_q + 1 WHEN cnt_q < 1625000 ELSE
           (OTHERS => '0');
  --povoleni citace pro rizeni diody
  led_cnt_en_i <= '1' WHEN cnt_q = 1625000 ELSE
                  '0';
  --citac pro rizeni diody
  led_cnt_d <= led_cnt_q + 1 WHEN led_cnt_en_i = '1' ELSE
               led_cnt_q;
  --vlastni rizeni diody
  led <= led_cnt_q(0) WHEN frq="00" ELSE
         led_cnt_q(1) WHEN frq="01" ELSE
			led_cnt_q(2) WHEN frq="10" ELSE
			led_cnt_q(3);
END ARCHITECTURE rtl;
