LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY ctrl IS 
  PORT (
    res        : IN  std_logic;
    clk        : IN  std_logic;
    
    dready     : IN  std_logic;
    done       : OUT std_logic;
    
    reg_sel    : OUT std_logic;
    reg_a_we   : OUT std_logic_vector (1 DOWNTO 0);
    reg_b_we   : OUT std_logic_vector (1 DOWNTO 0);
    alu_op_sel : OUT std_logic_vector (1 DOWNTO 0);
    dbus_op_sel: OUT std_logic_vector (1 DOWNTO 0)
  );
END ENTITY ctrl;

ARCHITECTURE rtl OF ctrl IS
  TYPE tstate IS (RESET, OFSET, INVERT, SECTI1, ADC, SECTI2, HOTOVO);
  SIGNAL curr_state  : tstate;
  SIGNAL next_state  : tstate;
  SIGNAL dready_sync : std_logic_vector (1 DOWNTO 0);
BEGIN

  state_reg : PROCESS (res, clk)
  BEGIN
    IF res = '1' THEN
      curr_state <= RESET;
    ELSIF clk = '1' AND clk'EVENT THEN
      curr_state <= next_state;
    END IF;
  END PROCESS state_reg;

  dready_sync_reg : PROCESS (res, clk)
  BEGIN
    IF res = '1' THEN
      dready_sync <= "00";
    ELSIF clk = '1' AND clk'EVENT THEN
      dready_sync <= dready_sync(0) & dready;
    END IF;
  END PROCESS dready_sync_reg;

  output_logic : PROCESS (curr_state)
  BEGIN
    done        <= '0';
    reg_sel     <= '0';
    reg_a_we    <= "00";
    reg_b_we    <= "00";
    alu_op_sel  <= "00";
    dbus_op_sel <= "11";
    CASE curr_state IS
      WHEN RESET  =>
      WHEN OFSET  =>
        reg_a_we    <= "01";
        dbus_op_sel <= "10";
      WHEN INVERT =>
        reg_a_we    <= "10";
        alu_op_sel  <= "00";
        reg_b_we    <= "01";
        dbus_op_sel <= "01";
      WHEN SECTI1 =>
        reg_a_we    <= "10";
        alu_op_sel  <= "01";
      WHEN ADC    =>
        reg_b_we    <= "01";
        dbus_op_sel <= "00";        
      WHEN SECTI2 =>
        reg_a_we    <= "10";
        alu_op_sel  <= "01";
      WHEN HOTOVO =>
        done        <= '1';
        reg_sel     <= '0';
    END CASE;
  END PROCESS output_logic;

  next_state_logic : PROCESS (dready_sync, curr_state)
  BEGIN
    next_state <= curr_state;
    CASE curr_state IS
      WHEN RESET  =>
        IF dready_sync(1) = '1' THEN
          next_state <= OFSET;
        END IF;
      WHEN OFSET  =>
        next_state <= INVERT;
      WHEN INVERT =>
        next_state <= SECTI1;
      WHEN SECTI1 =>
        next_state <= ADC;
      WHEN ADC    =>        
        next_state <= SECTI2;
      WHEN SECTI2 =>
        next_state <= HOTOVO;
      WHEN HOTOVO =>
        IF dready_sync(1) = '0' THEN
          next_state <= RESET;
        END IF;
    END CASE;
  END PROCESS next_state_logic;

END ARCHITECTURE rtl;
