LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY dp IS
  PORT (
    res         : IN  std_logic;
    clk         : IN  std_logic;
	 ad          : IN  std_logic_vector (7 DOWNTO 0);
	 offset      : IN  std_logic_vector (7 DOWNTO 0);
    dbus_op_sel : IN  std_logic_vector (1 DOWNTO 0);
    reg_sel     : IN  std_logic;
    reg_a_we    : IN  std_logic_vector (1 DOWNTO 0);
    reg_b_we    : IN  std_logic_vector (1 DOWNTO 0);
    alu_op_sel  : IN  std_logic_vector (1 DOWNTO 0);
    dbus        : OUT std_logic_vector (7 DOWNTO 0)
  );
END ENTITY dp;

ARCHITECTURE rtl OF dp IS
  SIGNAL a_d    : std_logic_vector (7 DOWNTO 0);
  SIGNAL a_q    : std_logic_vector (7 DOWNTO 0);
  SIGNAL b_d    : std_logic_vector (7 DOWNTO 0);
  SIGNAL b_q    : std_logic_vector (7 DOWNTO 0);  
  SIGNAL op_res : std_logic_vector (7 DOWNTO 0);
  SIGNAL in_op  : std_logic_vector (7 DOWNTO 0);
BEGIN

  in_op <= ad     WHEN dbus_op_sel = "00" ELSE
           offset WHEN dbus_op_sel = "10" ELSE
			  x"01";

  mux_ab : PROCESS (op_res, reg_a_we, reg_b_we, in_op, a_q, b_q)  
  BEGIN
    IF    reg_a_we = "00" THEN
      a_d <= a_q;
    ELSIF reg_a_we = "01" THEN
      a_d <= in_op;
    ELSIF reg_a_we = "10" THEN
      a_d <= op_res;
    ELSE --11 nebo jina kombinace
      a_d <= b_q;
    END IF;
    IF    reg_b_we = "00" THEN
      b_d <= b_q;
    ELSIF reg_b_we = "01" THEN
      b_d <= in_op;
    ELSIF reg_b_we = "11" THEN
      b_d <= a_q;
    ELSE --10 nebo jina kombinace
      b_d <= b_q;
    END IF;
  END PROCESS mux_ab;

  reg_ab : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN
      a_q <= (OTHERS => '0');
      b_q <= (OTHERS => '0');
    ELSIF clk'EVENT AND clk = '1' THEN
      a_q <= a_d;
      b_q <= b_d;
    END IF;
  END PROCESS reg_ab;

  mux_op_res : PROCESS (a_q, b_q, alu_op_sel)
  BEGIN
    CASE alu_op_sel IS
      WHEN "00"   =>
        op_res <= NOT(a_q);
      WHEN "01"   =>
        op_res <= a_q + b_q;
      WHEN "10"   =>
        op_res <= a_q AND b_q;
      WHEN "11"   =>
        op_res <= a_q OR b_q;
      WHEN OTHERS =>
        op_res <= (OTHERS => 'X');
    END CASE;
  END PROCESS mux_op_res;

  dbus <= a_q WHEN reg_sel = '0' ELSE b_q;

END ARCHITECTURE rtl;
