LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;
LIBRARY std_developerskit;
--USE std_developerskit.std_iopak.all;

ENTITY testbench IS
BEGIN
END ENTITY testbench;

ARCHITECTURE behav OF testbench IS

  COMPONENT top_level IS
    PORT (
      clk        : IN  std_logic;
      res        : IN  std_logic;

      dready     : IN  std_logic;
      done       : OUT std_logic;

      dbus       : INOUT std_logic_vector (7 DOWNTO 0);
      offset     : IN    std_logic_vector (7 DOWNTO 0);
      ad         : IN    std_logic_vector (7 DOWNTO 0)
    );
  END COMPONENT top_level;

  SIGNAL clk_i      : std_logic;
  SIGNAL res_i      : std_logic;
  SIGNAL dready_i   : std_logic;
  SIGNAL done_i     : std_logic;
  SIGNAL dbus_i     : std_logic_vector (7 DOWNTO 0);
  SIGNAL offset_i   : std_logic_vector (7 DOWNTO 0);
  SIGNAL ad_i       : std_logic_vector (7 DOWNTO 0);
  
  CONSTANT clk_per     : time    := 100 ns; --perioda hodin
  CONSTANT clk_timeout : natural := 20; --max 20 clk pulzu mezi dready a done 

BEGIN
--------------------------------------------------------------------------------
-- Seznam pozadavku ktere navrh musi splnovat - verifikacni plan. Seznam je
-- extrahovany ze specifikace.
--------------------------------------------------------------------------------
-- 1) handshake dready - done funguje podle specifikace (obvod vzdy odpovi do
--    urcite doby, nikdy nevygeneruje done sam od sebe, atd.). 
-- 2) kompenzace ofsetu pracuje spravne, vstupni hodnoty jsou cteny z odpovi-
--    dajicich sbernic, vystupni hodnota je zapsana ve spravnem okamziku, atd.
--------------------------------------------------------------------------------

  design_under_test : top_level
    PORT MAP (
      clk        => clk_i,
      res        => res_i,
      dready     => dready_i,
      done       => done_i,
      dbus       => dbus_i,
      offset     => offset_i,
      ad         => ad_i
    );

  testbench_control : PROCESS

    PROCEDURE gen_clk (CONSTANT pulses : IN natural := 1) IS
    BEGIN
      FOR pulse_no IN 0 TO pulses - 1 LOOP
        clk_i <= '0';
        WAIT FOR clk_per / 2;
        clk_i <= '1';
        WAIT FOR clk_per / 2;
      END LOOP;
    END PROCEDURE gen_clk;
 
    PROCEDURE execute_one_cycle (CONSTANT ad     : IN integer;
                                 CONSTANT offset : IN integer;
                                 CONSTANT cyklu  : IN integer ) IS
      VARIABLE ad_vstup     : std_logic_vector (7 DOWNTO 0);
      VARIABLE offset_vstup : std_logic_vector (7 DOWNTO 0);      
      VARIABLE komp_vstup   : std_logic_vector (7 DOWNTO 0);
    BEGIN
      --konvertuj integer hodnoty na std_logic_vectory
      ad_vstup     := std_logic_vector(conv_signed(ad    , 8));
      offset_vstup := std_logic_vector(conv_signed(offset, 8));
      komp_vstup   := ad_vstup - offset_vstup;
      --prived hodnoty na vstup obvodu
      ad_i     <= ad_vstup;
      offset_i <= offset_vstup;
      --aktivuj handshake
      dready_i <= '1';
      --generuj hodinove pulzy dokud neprijde done a dokud neprekrocis timeout -
      --  pak se obvod patrne nekde zasekl a bude v nem chyba
      FOR clk_cycle IN 0 TO clk_timeout-1 LOOP
        gen_clk;
        IF done_i = '1' THEN --done v log. 1 - konec
          EXIT;
        END IF;
      END LOOP;
      --pokud zde neni done v log. 1, vyprsel cas
      ASSERT done_i = '1'
      REPORT "tesbench.vhd: timeout - obvod nereaguje vcas!"
      SEVERITY FAILURE;
      --zkontroluj vystupni sbernici - kompenzovanou hodnotu
      ASSERT dbus_i = komp_vstup
      REPORT "tesbench.vhd: chybny vypocet - na vystupu je " & 
              natural'IMAGE(conv_integer(unsigned(dbus_i))) & " misto " &
              natural'IMAGE(conv_integer(unsigned(komp_vstup))) & "!"
      SEVERITY FAILURE;
      --vygeneruj nekolik cyklu hodin - overeni ze obvod ceka na spadovou hranu 
      --dready
      FOR clk_cycle IN 0 TO cyklu + 2 LOOP
        --zkontroluj spravny handshake - dready pujde do log. 0
        IF clk_cycle = cyklu THEN
          dready_i <= '0';
        END IF; 
        ASSERT done_i = '1'
        REPORT "tesbench.vhd: done spadlo do log. 0!"
        SEVERITY FAILURE;
        gen_clk;
      END LOOP;
      ASSERT done_i = '0'
      REPORT "tesbench.vhd: done nespadlo do log. 0!"
      SEVERITY FAILURE;
      --konec, oba ridici signaly jsou v log. 0
    END PROCEDURE execute_one_cycle;
 
  BEGIN
    --vyresetuj obvod a nastav I/O signaly do def. hodnot
    res_i    <= '1';
    dready_i <= '0';
    ad_i     <= "00000000";
    offset_i <= "00000000";
    dbus_i   <= "ZZZZZZZZ"; --nebudime, pouze cteme
    WAIT FOR 100 ns;
    res_i <= '0';
    gen_clk (5); --dready=0 takze se nic nedeje, obvod nesmi nic udelat
    execute_one_cycle(121, 10,  5);
    execute_one_cycle(127,100, 12);
    execute_one_cycle(-15, 28,  1);
    execute_one_cycle(-78,-30, 30);
    execute_one_cycle( 69, 46,  0);
    ASSERT FALSE
    REPORT "Simulace neodhalila zadne chyby"
    SEVERITY NOTE;
    WAIT;
  END PROCESS testbench_control;

END ARCHITECTURE behav;  
