LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY top_level IS
  PORT (
    clk        : IN  std_logic;
    res        : IN  std_logic;

    dready     : IN  std_logic;
    done       : OUT std_logic;

    dbus       : OUT std_logic_vector (7 DOWNTO 0);
    offset     : IN  std_logic_vector (7 DOWNTO 0);
    ad         : IN  std_logic_vector (7 DOWNTO 0)
  );
END ENTITY top_level;

ARCHITECTURE rtl OF top_level IS
  SIGNAL reg_sel_i     : std_logic;
  SIGNAL reg_a_we_i    : std_logic_vector (1 DOWNTO 0);
  SIGNAL reg_b_we_i    : std_logic_vector (1 DOWNTO 0);
  SIGNAL alu_op_sel_i  : std_logic_vector (1 DOWNTO 0);
  SIGNAL dbus_op_sel_i : std_logic_vector (1 DOWNTO 0);
  SIGNAL dbus_i        : std_logic_vector (7 DOWNTO 0);

  COMPONENT dp IS
    PORT (
      res         : IN  std_logic;
      clk         : IN  std_logic;
	   ad          : IN  std_logic_vector (7 DOWNTO 0);
	   offset      : IN  std_logic_vector (7 DOWNTO 0);
      dbus_op_sel : IN  std_logic_vector (1 DOWNTO 0);
      reg_sel     : IN  std_logic;
      reg_a_we    : IN  std_logic_vector (1 DOWNTO 0);
      reg_b_we    : IN  std_logic_vector (1 DOWNTO 0);
      alu_op_sel  : IN  std_logic_vector (1 DOWNTO 0);
      dbus        : OUT std_logic_vector (7 DOWNTO 0)
    );
  END COMPONENT dp;

  COMPONENT ctrl IS 
    PORT (
      clk        : IN  std_logic;
      res        : IN  std_logic;
    
      dready     : IN  std_logic;
      done       : OUT std_logic;
    
      reg_sel    : OUT std_logic;
      reg_a_we   : OUT std_logic_vector (1 DOWNTO 0);
      reg_b_we   : OUT std_logic_vector (1 DOWNTO 0);
      alu_op_sel : OUT std_logic_vector (1 DOWNTO 0);
      dbus_op_sel: OUT std_logic_vector (1 DOWNTO 0)
    );
  END COMPONENT ctrl;

BEGIN

  i_dp : dp
    PORT MAP (
      res         => res,
      clk         => clk,
	   ad          => ad,
		offset      => offset,
		dbus_op_sel => dbus_op_sel_i,
      reg_sel     => reg_sel_i,
      reg_a_we    => reg_a_we_i,
      reg_b_we    => reg_b_we_i,
      alu_op_sel  => alu_op_sel_i,
      dbus        => dbus
    );

  i_ctrl : ctrl
    PORT MAP (
      clk        => clk,
      res        => res,
      dready     => dready,
      done       => done,
      reg_sel    => reg_sel_i,
      reg_a_we   => reg_a_we_i,
      reg_b_we   => reg_b_we_i,
      alu_op_sel => alu_op_sel_i,
      dbus_op_sel=> dbus_op_sel_i
    );

END rtl;
