LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY arith_opers IS
  GENERIC (
    k          : natural := 1;
    N          : natural := 8
  );
  PORT (
    res        : IN std_logic;
    clk        : IN std_logic;

    a          : IN    signed   (N-1   DOWNTO 0);
    b          : IN    signed   (N-1   DOWNTO 0);
    a_abs      : OUT   unsigned (N-1   DOWNTO 0);
    c_add      : OUT   signed   (N-1   DOWNTO 0);
    c_sat_add  : OUT   signed   (N-1   DOWNTO 0);
    c_ext_add  : OUT   signed   (N     DOWNTO 0);
    c_us_mult  : OUT   unsigned (2*N-1 DOWNTO 0);
    c_s_mult   : OUT   signed   (2*N-1 DOWNTO 0);
    c_us_rshift: OUT   unsigned (N-1   DOWNTO 0);
    c_s_rshift : OUT   signed   (N-1   DOWNTO 0)
  );
END ENTITY arith_opers;

ARCHITECTURE rtl OF arith_opers IS
  SIGNAL c_sat_add_i : signed   (7 DOWNTO 0);
  SIGNAL overflow_i  : std_logic;
BEGIN
  --scitacka
  c_add       <= a + b;
  --scitacka se znamenkovym rozsirenim
  c_ext_add   <= (a(N-1)&a) + (b(N-1)&b);
  --scitacka se saturaci, priklad je z duvodu snadne pochopitelnosti
  --jen pro osmibitova cisla
  c_sat_add_i <= a + b;
  overflow_i  <= '1' WHEN a(N-1) = b(N-1) AND c_sat_add_i(N-1) /= a(N-1) ELSE '0';
  c_sat_add   <= "01111111" WHEN overflow_i = '1' AND a(7) = '0' ELSE 
                 "10000000" WHEN overflow_i = '1' AND a(7) = '1' ELSE 
                 c_sat_add_i;
  --absolutni hodnota
  a_abs       <= unsigned(a) WHEN a(N-1) = '0' ELSE
                 unsigned(NOT(a))+1;
  --nasobeni bez znamenka
  c_us_mult   <= unsigned(a) * unsigned(b);
  --nasobeni se znamenkem
  c_s_mult    <= a * b;
  --posuv vpravo bez znamenka
  c_us_rshift(N-1 DOWNTO N-k) <= (OTHERS => '0'); 
  c_us_rshift(N-k-1 DOWNTO 0) <= unsigned(a(N-1 DOWNTO k));
  --posuv vpravo se znamenkem
  c_s_rshift (N-1 DOWNTO N-k) <= (OTHERS => a(N-1)); 
  c_s_rshift (N-k-1 DOWNTO 0) <= a(N-1 DOWNTO k);
END ARCHITECTURE rtl;
