LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY testbench IS
  GENERIC (
      k          : natural := 1;
      N          : natural := 8
  );
BEGIN
END ENTITY testbench;

ARCHITECTURE behav OF testbench IS

  COMPONENT arith_opers IS
    GENERIC (
      k          : natural;
      N          : natural
    );
    PORT (
      res        : IN std_logic;
      clk        : IN std_logic;

      a          : IN    signed   (N-1   DOWNTO 0);
      b          : IN    signed   (N-1   DOWNTO 0);
      a_abs      : OUT   unsigned (N-1   DOWNTO 0);
      c_add      : OUT   signed   (N-1   DOWNTO 0);
      c_sat_add  : OUT   signed   (N-1   DOWNTO 0);
      c_ext_add  : OUT   signed   (N     DOWNTO 0);
      c_us_mult  : OUT   unsigned (2*N-1 DOWNTO 0);
      c_s_mult   : OUT   signed   (2*N-1 DOWNTO 0);
      c_us_rshift: OUT   unsigned (N-1   DOWNTO 0);
      c_s_rshift : OUT   signed   (N-1   DOWNTO 0)
    );
  END COMPONENT arith_opers;

  SIGNAL clk_i         : std_logic;
  SIGNAL res_i         : std_logic;
  SIGNAL a_i           : signed   (N-1   DOWNTO 0);
  SIGNAL b_i           : signed   (N-1   DOWNTO 0);
  SIGNAL c_add_i       : signed   (N-1   DOWNTO 0);
  SIGNAL c_sat_add_i   : signed   (N-1   DOWNTO 0);
  SIGNAL c_ext_add_i   : signed   (N     DOWNTO 0);
  SIGNAL a_abs_i       : unsigned (N-1   DOWNTO 0);
  SIGNAL c_us_mult_i   : unsigned (2*N-1 DOWNTO 0);
  SIGNAL c_s_mult_i    : signed   (2*N-1 DOWNTO 0);
  SIGNAL c_us_rshift_i : unsigned (N-1   DOWNTO 0);
  SIGNAL c_s_rshift_i  : signed   (N-1   DOWNTO 0);
  
  CONSTANT clk_per     : time    := 100 ns; --perioda hodin 

BEGIN

  design_under_test : arith_opers
    GENERIC MAP (
      k          => k,
      N          => N
    )
    PORT MAP (
      clk         => clk_i,
      res         => res_i,

      a           => a_i,
      b           => b_i,

      a_abs       => a_abs_i,
      c_add       => c_add_i,
      c_sat_add   => c_sat_add_i,
      c_ext_add   => c_ext_add_i,
      c_us_mult   => c_us_mult_i,
      c_s_mult    => c_s_mult_i,
      c_us_rshift => c_us_rshift_i,
      c_s_rshift  => c_s_rshift_i       
    );

  testbench_control : PROCESS

    PROCEDURE gen_clk (CONSTANT pulses : IN natural := 1) IS
    BEGIN
      FOR pulse_no IN 0 TO pulses - 1 LOOP
        clk_i <= '0';
        WAIT FOR clk_per / 2;
        clk_i <= '1';
        WAIT FOR clk_per / 2;
      END LOOP;
    END PROCEDURE gen_clk;
 
    PROCEDURE execute_one_cycle (CONSTANT a      : IN integer;
                                 CONSTANT b      : IN integer) IS
      VARIABLE a_vstup          : signed   (N-1   DOWNTO 0);
      VARIABLE b_vstup          : signed   (N-1   DOWNTO 0);
      --ocekavane vystupy
      VARIABLE c_add_ocek       : signed   (N-1   DOWNTO 0);
      VARIABLE c_sat_add_ocek   : signed   (N-1   DOWNTO 0);
      VARIABLE c_ext_add_ocek   : signed   (N     DOWNTO 0);
      VARIABLE c_s_mult_ocek    : signed   (2*N-1 DOWNTO 0);
      VARIABLE a_abs_ocek       : unsigned (N-1   DOWNTO 0);

      VARIABLE a_us             : natural; --unsigned a
      VARIABLE b_us             : natural; --unsigned a
      VARIABLE c_us_mult_ocek   : unsigned (2*N-1 DOWNTO 0);
      VARIABLE c_us_rshift_ocek : unsigned (N-1   DOWNTO 0);
      VARIABLE c_s_rshift_ocek  : signed   (N-1   DOWNTO 0);
    BEGIN
      --konvertuj integer hodnoty na std_logic_vectory
      a_vstup        := to_signed  (a     , N  );
      b_vstup        := to_signed  (b     , N  );
      c_add_ocek     := to_signed  (a+b   , N  );
      c_ext_add_ocek := to_signed  (a+b   , N+1);
      a_abs_ocek     := to_unsigned(abs(a), N  );
      c_s_mult_ocek  := to_signed  (a*b   , 2*N);
      --nasobeni bez znamenka, vstupni operandy nejsou v 2's complement
      IF a < 0 THEN
        a_us := a + 2**N;
      ELSE
        a_us := a;
      END IF;
      IF b < 0 THEN
        b_us := b + 2**N;
      ELSE
        b_us := b;
      END IF;          
      c_us_mult_ocek   := to_unsigned(a_us*b_us, 2*N);
      c_us_rshift_ocek := to_unsigned(a_us/2**k,   N);
      --simulace saturace
      IF    a+b >  2**(N-1)-1 THEN
        c_sat_add_ocek := to_signed(+2**(N-1)-1, N);
      ELSIF a+b < -2**(N-1)   THEN
        c_sat_add_ocek := to_signed(-2**(N-1)  , N);
      ELSE
        c_sat_add_ocek := to_signed(a+b        , N);
      END IF;
      --prived hodnoty na vstup obvodu
      a_i <= a_vstup;
      b_i <= b_vstup;
      --vygeneruj jeden hodinovy pulz - pro operatory s registrem
      gen_clk;
      --zkontroluj vystupni sbernice
      ASSERT c_add_i = c_add_ocek
      REPORT "tesbench.vhd: chybny soucet - na vystupu je " & 
              integer'IMAGE(to_integer(c_add_i)) & " misto " &
              integer'IMAGE(to_integer(c_add_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT c_ext_add_ocek = c_ext_add_i
      REPORT "tesbench.vhd: chybny soucet se zn. rozsirenim - na vystupu je " & 
              integer'IMAGE(to_integer(c_ext_add_i)) & " misto " &
              integer'IMAGE(to_integer(c_ext_add_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT c_sat_add_ocek = c_sat_add_i
      REPORT "tesbench.vhd: chybny soucet se saturaci - na vystupu je " & 
              integer'IMAGE(to_integer(c_sat_add_i)) & " misto " &
              integer'IMAGE(to_integer(c_sat_add_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT c_us_mult_ocek = c_us_mult_i
      REPORT "tesbench.vhd: chybny soucin bez znamenka - na vystupu je " & 
              integer'IMAGE(to_integer(c_us_mult_i)) & " misto " &
              integer'IMAGE(to_integer(c_us_mult_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT c_s_mult_ocek = c_s_mult_i
      REPORT "tesbench.vhd: chybny soucin se znamenkem - na vystupu je " & 
              integer'IMAGE(to_integer(c_s_mult_i)) & " misto " &
              integer'IMAGE(to_integer(c_s_mult_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT a_abs_i = a_abs_ocek
      REPORT "tesbench.vhd: chybny vypocet abs. hodnoty - na vystupu je " & 
              integer'IMAGE(to_integer(a_abs_i)) & " misto " &
              integer'IMAGE(to_integer(a_abs_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT c_us_rshift_ocek = c_us_rshift_i
      REPORT "tesbench.vhd: chybny posuv vpravo bez znamenka - na vystupu je " & 
              integer'IMAGE(to_integer(c_us_rshift_i)) & " misto " &
              integer'IMAGE(to_integer(c_us_rshift_ocek)) & "!"
      SEVERITY FAILURE;
      --konec
    END PROCEDURE execute_one_cycle;
 
  BEGIN
    --vyresetuj obvod a nastav I/O signaly do def. hodnot
    res_i    <= '1';
    WAIT FOR 100 ns;
    res_i <= '0';
    FOR smycka_a IN -15 TO 15 LOOP
      FOR smycka_b IN -15 TO 15 LOOP
        execute_one_cycle(smycka_a*3, smycka_b*6);
      END LOOP;
    END LOOP;
    ASSERT FALSE
    REPORT "Simulace neodhalila zadne chyby"
    SEVERITY NOTE;
    WAIT;
  END PROCESS testbench_control;

END ARCHITECTURE behav;  
