LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY odmocnina IS
  GENERIC (
    N          : natural := 8
  );
  PORT (
    res        : IN  std_logic;
    clk        : IN  std_logic;
    
    load       : IN  std_logic; --inicializace vypoctu
    sqrt_en    : IN  std_logic; --povoleni vypoctu jednoho bitu odmocniny

    a          : IN  unsigned (2*N-1   DOWNTO 0);--odmocnenec
   
    x          : OUT unsigned (  N-1   DOWNTO 0);--odmocnina
    r          : OUT unsigned (  N     DOWNTO 0) --zbytek
  );
END ENTITY odmocnina;

ARCHITECTURE rtl OF odmocnina IS
  SIGNAL a_d         : unsigned (2*N-1 DOWNTO 0);
  SIGNAL a_q         : unsigned (2*N-1 DOWNTO 0);
  SIGNAL x_d         : unsigned (  N-1 DOWNTO 0);
  SIGNAL x_q         : unsigned (  N-1 DOWNTO 0);
  SIGNAL r_d         : unsigned (  N+1 DOWNTO 0);
  SIGNAL r_q         : unsigned (  N+1 DOWNTO 0);
  
  SIGNAL r_i         : unsigned (  N+1 DOWNTO 0);
  SIGNAL x_i         : unsigned (  N+1 DOWNTO 0);
BEGIN

  registers : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN
      r_q <= (OTHERS => '0');
      x_q <= (OTHERS => '0');
      a_q <= (OTHERS => '0');
    ELSIF clk='1' AND clk'EVENT THEN
      r_q <= r_d;
      x_q <= x_d;
      a_q <= a_d;
    END IF; 
  END PROCESS registers;

  --Odmocnenec je zapsan na zacatku operace a posouvan o dva bity 
  --v kazdem cyklu.
  a_d <= a                          WHEN load    = '1' ELSE
         a_q(2*N-3 DOWNTO 0) & "00" WHEN sqrt_en = '1' ELSE
         a_q;
        
  --Odmocnina - vysledek - je vynulovan na zacatku operace a posouvan o bit 
  --v kazdem cyklu.
  x_d <= (OTHERS => '0')            WHEN load    = '1' ELSE
         x_q(  N-2 DOWNTO 0) &  "1" WHEN sqrt_en = '1' AND r_d(N+1) = '0' ELSE
         x_q(  N-2 DOWNTO 0) &  "0" WHEN sqrt_en = '1' AND r_d(N+1) = '1' ELSE
         x_q;

  --Hodnota odmocniny upravena pred scitanim/odecitanim v kroku 3 v algoritmu
  x_i <= x_q & "11" WHEN r_q(N+1) = '1' ELSE
         NOT(x_q & "01") + 1;
  
  --Zbytek se sepsanymi dalsimi dvema cislicemi
  r_i <= r_q(N-1 DOWNTO 0)&a_q(2*N-1)&a_q(2*N-2);
  
  --Operace scitani; zbytek ale aktualizujeme az kdyz je treba 
  --provest posuv operandu
  r_d <= x_i + r_i       WHEN sqrt_en = '1' ELSE
         (OTHERS => '0') WHEN load    = '1' ELSE
         r_q;

  x <= x_q;
  r <= r_q(N DOWNTO 0);  

END ARCHITECTURE rtl;
