LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE IEEE.math_real.ALL;

ENTITY testbench IS
  GENERIC (
      N          : natural := 8
  );
BEGIN
END ENTITY testbench;

ARCHITECTURE behav OF testbench IS

  COMPONENT odmocnina IS
  GENERIC (
    N          : natural := 8
  );
  PORT (
    res        : IN  std_logic;
    clk        : IN  std_logic;
    
    load       : IN  std_logic; --inicializace vypoctu
    sqrt_en    : IN  std_logic; --povoleni vypoctu jednoho bitu odmocniny

    a          : IN  unsigned (2*N-1   DOWNTO 0);--odmocnenec

    x          : OUT unsigned (  N-1   DOWNTO 0);--odmocnina
    r          : OUT unsigned (  N     DOWNTO 0) --zbytek
  );
  END COMPONENT odmocnina;

  SIGNAL clk_i         : std_logic;
  SIGNAL res_i         : std_logic;
 
  SIGNAL a_i           : unsigned (2*N-1   DOWNTO 0);--odmocnenec
  SIGNAL x_i           : unsigned (  N-1   DOWNTO 0);--odmocnina
  SIGNAL r_i           : unsigned (  N     DOWNTO 0);--zbytek
  SIGNAL load_i        : std_logic;
  SIGNAL sqrt_en_i     : std_logic;

  CONSTANT clk_per     : time    := 100 ns; --perioda hodin 

BEGIN

  design_under_test : odmocnina
    GENERIC MAP (
      N          => N
    )
    PORT MAP (
      clk         => clk_i,
      res         => res_i,

      load        => load_i,
      sqrt_en     => sqrt_en_i,

      a           => a_i,
      x           => x_i,
      r           => r_i
    );

  testbench_control : PROCESS

    PROCEDURE gen_clk (CONSTANT pulses : IN natural := 1) IS
    BEGIN
      FOR pulse_no IN 0 TO pulses - 1 LOOP
        clk_i <= '0';
        WAIT FOR clk_per / 2;
        clk_i <= '1';
        WAIT FOR clk_per / 2;
      END LOOP;
    END PROCEDURE gen_clk;
 
    PROCEDURE execute_sqrt      (CONSTANT a : IN natural) IS
      VARIABLE a_vstup          : unsigned (2*N-1   DOWNTO 0);
      --ocekavane vystupy
      VARIABLE x_num            : natural;
      VARIABLE x_ocek           : unsigned (  N-1   DOWNTO 0);
    BEGIN
      --konvertuj integer hodnoty na std_logic_vectory
      a_vstup        := to_unsigned(a,2*N);
      x_num          := natural(trunc(sqrt(real(a))));
      x_ocek         := to_unsigned(x_num,N);
      --prived hodnoty na vstup obvodu
      a_i <= a_vstup;

      --vygeneruj jeden hodinovy pulz - nacteni hodnot
      load_i <= '1';
      gen_clk;
      load_i <= '0';
     
      --vygeneruj N hodinovych pulzu pro vypocet
      sqrt_en_i <= '1';
      FOR cyklus IN N-1 DOWNTO 0 LOOP
        gen_clk;
      END LOOP;
      sqrt_en_i <= '0';
     
      --zkontroluj vystupni sbernici
      ASSERT x_ocek = x_i
      REPORT "tesbench.vhd: chybny vysledek odmocniny " & 
              integer'IMAGE(to_integer(x_i)) & " misto " &
              integer'IMAGE(to_integer(x_ocek)) & "!"
      SEVERITY FAILURE;
      ASSERT false
      REPORT "tesbench.vhd: odmocnina z  " & natural'IMAGE(a) & 
               " = " & natural'IMAGE(x_num) & "."
      SEVERITY NOTE;

      --konec
    END PROCEDURE execute_sqrt;
 
  BEGIN
    --vyresetuj obvod a nastav I/O signaly do def. hodnot
    sqrt_en_i <= '0';
    load_i    <= '0';
    res_i     <= '1';
    WAIT FOR 100 ns;
    res_i     <= '0';

    FOR smycka_a IN 500 DOWNTO 0 LOOP
        execute_sqrt(smycka_a*100);
    END LOOP;
    ASSERT FALSE
    REPORT "Simulace neodhalila zadne chyby"
    SEVERITY NOTE;
    WAIT;
  END PROCESS testbench_control;

END ARCHITECTURE behav;  
