library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

ENTITY dp IS 
  GENERIC (
    n : natural := 16
  );
  PORT (
    clk    : IN  std_logic;
    res    : IN  std_logic;

    a_in   : IN  unsigned (n-1 downto 0);
    b_in   : IN  unsigned (n-1 downto 0);
    we_a   : IN  std_logic_vector (1 DOWNTO 0);
    we_b   : IN  std_logic;
    we_c   : IN  std_logic_vector (1 downto 0);
    c_out  : OUT unsigned (n-1 downto 0);
    a_gt0  : OUT std_logic
	);
END dp;

ARCHITECTURE rtl OF dp IS
  SIGNAL a_q     : unsigned (n-1 DOWNTO 0);
  SIGNAL a_d     : unsigned (n-1 DOWNTO 0);  
  SIGNAL b_q     : unsigned (n-1 DOWNTO 0);
  SIGNAL b_d     : unsigned (n-1 DOWNTO 0);
  SIGNAL c_q     : unsigned (n-1 DOWNTO 0);
  SIGNAL c_d     : unsigned (n-1 DOWNTO 0);
  SIGNAL a_dec_i : unsigned (n-1 DOWNTO 0);
  SIGNAL a_gt0_q : std_logic;
  SIGNAL a_gt0_d : std_logic;
BEGIN

  reg_variables : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN
	   a_q     <= (OTHERS => '0');
	   b_q     <= (OTHERS => '0');
	   c_q     <= (OTHERS => '0');
		a_gt0_q <= '0';
	 ELSIF clk'EVENT AND clk='1' THEN
	   a_q     <= a_d;
		b_q     <= b_d;
		c_q     <= c_d;
		a_gt0_q <= a_gt0_d;
	 END IF;
  END PROCESS reg_variables;

  a_dec_i <= a_q - 1;
  a_d     <= a_in            WHEN we_a = "01" ELSE
             a_dec_i         WHEN we_a = "10" ELSE
			    a_q;
  b_d     <= b_in            WHEN we_b = '1'  ELSE b_q;
  c_d     <= (OTHERS => '0') WHEN we_c = "01" ELSE
			    c_q + b_q       WHEN we_c = "10" ELSE
			    c_q;
  
  a_gt0_d <= '0' WHEN a_q     = 0 ELSE
				 '1';
  
  c_out   <= c_q;
  a_gt0   <= a_gt0_q;
  
END ARCHITECTURE rtl;