LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY tb IS
END ENTITY tb;

ARCHITECTURE behav OF tb IS

  CONSTANT clk_per : time    := 100 ns; --perioda hodin 
  CONSTANT n       : natural := 16;

  COMPONENT dp IS 
    GENERIC (
      n : natural := 16
    );
    PORT (
      clk    : IN  std_logic;
      res    : IN  std_logic;

      a_in   : IN  unsigned (n-1 downto 0);
      b_in   : IN  unsigned (n-1 downto 0);
      we_a   : IN  std_logic_vector (1 DOWNTO 0);
      we_b   : IN  std_logic;
      we_c   : IN  std_logic_vector (1 downto 0);
      c_out  : OUT unsigned (n-1 downto 0);
      a_gt0  : OUT std_logic
	  );
  END COMPONENT dp;

  SIGNAL clk_i   : std_logic;
  SIGNAL res_i   : std_logic;
  SIGNAL a_in_i  : unsigned (n-1 DOWNTO 0);
  SIGNAL b_in_i  : unsigned (n-1 DOWNTO 0);
  SIGNAL we_a_i  : std_logic_vector (1 DOWNTO 0);
  SIGNAL we_b_i  : std_logic;
  SIGNAL we_c_i  : std_logic_vector (1 DOWNTO 0);
  SIGNAL a_gt0_i : std_logic;
  SIGNAL c_out_i : unsigned (n-1 DOWNTO 0);

BEGIN

  i_dp : dp 
    GENERIC MAP (
      n => n
    )
    PORT MAP (
      clk    => clk_i,
      res    => res_i,

      a_in   => a_in_i,
      b_in   => b_in_i,
      we_a   => we_a_i,
      we_b   => we_b_i,
      we_c   => we_c_i,
      c_out  => c_out_i,
      a_gt0  => a_gt0_i
	  );

  
  testbench_control : PROCESS

    PROCEDURE gen_clk (CONSTANT pulses : IN natural := 1) IS
    BEGIN
      FOR pulse_no IN 0 TO pulses - 1 LOOP
        clk_i <= '0';
        WAIT FOR clk_per / 2;
        clk_i <= '1';
        WAIT FOR clk_per / 2;
      END LOOP;
    END PROCEDURE gen_clk;
 
    PROCEDURE execute_one_cycle (CONSTANT a      : IN natural;
                                 CONSTANT b      : IN natural) IS
      VARIABLE a_vstup          : unsigned   (N-1   DOWNTO 0);
      VARIABLE b_vstup          : unsigned   (N-1   DOWNTO 0);
      --ocekavane vystupy
      VARIABLE c_s_mult_ocek    : unsigned   (2*N-1 DOWNTO 0);
    BEGIN
      --konvertuj integer hodnoty na std_logic_vectory
      a_vstup        := to_unsigned  (a     , N  );
      b_vstup        := to_unsigned  (b     , N  );
      c_s_mult_ocek  := to_unsigned  (a*b   , 2*N);
      --prived hodnoty na vstup obvodu
      a_in_i <= a_vstup;
      b_in_i <= b_vstup;
		--zapis vstupni hodnoty do registru
		we_a_i <= "01";
		we_b_i <= '1';
		we_c_i <= "01";
      --vygeneruj jeden hodinovy pulz - zapis do registru
      gen_clk;
		--dalsi hodinovy pulz pro aktualizaci a_gt0 priznaku
		we_a_i <= "00";
		we_b_i <= '0';
		we_c_i <= "00";
      gen_clk;
      WHILE a_gt0_i = '1' LOOP
		  --c=c+b,a=a-1
        we_a_i <= "10";
	     we_b_i <= '0';
		  we_c_i <= "10";
		  gen_clk;
        --aktualizuj a_gt0 priznak		  
		  we_a_i <= "00";
		  we_b_i <= '0';
		  we_c_i <= "00";
        gen_clk;
      END LOOP;		
		
      --zkontroluj vystupni sbernice
      ASSERT c_s_mult_ocek = c_out_i
      REPORT "tesbench.vhd: chybny soucin - na vystupu je " & 
              integer'IMAGE(to_integer(c_out_i)) & " misto " &
              integer'IMAGE(to_integer(c_s_mult_ocek)) & "!"
      SEVERITY FAILURE;
      --konec
    END PROCEDURE execute_one_cycle;
 
  BEGIN
    --vyresetuj obvod a nastav I/O signaly do def. hodnot
    res_i    <= '1';
	 we_a_i   <= "00";
	 we_b_i   <= '0';
	 we_c_i   <= "00";
    WAIT FOR 100 ns;
    res_i <= '0';
	 --nacti hodnoty

    FOR smycka_a IN 0 TO 15 LOOP
      FOR smycka_b IN 0 TO 15 LOOP
        execute_one_cycle(smycka_a*7, smycka_b*7);
      END LOOP;
    END LOOP;
    ASSERT FALSE
    REPORT "Simulace neodhalila zadne chyby"
    SEVERITY NOTE;
    WAIT;
  END PROCESS testbench_control;
  
END ARCHITECTURE behav;