library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.ALL;

ENTITY dp IS 
  GENERIC (
    n : natural := 16
  );
  PORT (
    clk        : IN  std_logic;
    res        : IN  std_logic;

    a_in       : IN  unsigned (n-1 downto 0);
    we_lo      : IN  std_logic_vector (1 DOWNTO 0);
    we_hi      : IN  std_logic_vector (1 DOWNTO 0);
    sq_lo_out  : OUT unsigned (n-1 downto 0);
    gt1        : OUT std_logic;
    gta        : OUT std_logic
	);
END dp;

ARCHITECTURE rtl OF dp IS
  SIGNAL a_q     : unsigned (  n-1 DOWNTO 0);
  SIGNAL a_d     : unsigned (  n-1 DOWNTO 0);  
  SIGNAL sq_lo_q : unsigned (  n-1 DOWNTO 0);
  SIGNAL sq_lo_d : unsigned (  n-1 DOWNTO 0);
  SIGNAL sq_hi_q : unsigned (  n-1 DOWNTO 0);
  SIGNAL sq_hi_d : unsigned (  n-1 DOWNTO 0);
  SIGNAL sq_tmp  : unsigned (  n-1 DOWNTO 0);
  SIGNAL sq2_tmp : unsigned (2*n-1 DOWNTO 0);
  SIGNAL gt1_q   : std_logic;
  SIGNAL gt1_d   : std_logic;
  SIGNAL gta_q   : std_logic;
  SIGNAL gta_d   : std_logic;
BEGIN

  reg_variables : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN
	   a_q     <= (OTHERS => '0');
	   sq_lo_q <= (OTHERS => '0');
	   sq_hi_q <= (OTHERS => '0');
	   gt1_q   <= '0';
	   gta_q   <= '0';
	 ELSIF clk'EVENT AND clk='1' THEN
	   a_q     <= a_d;
	   sq_lo_q <= sq_lo_d;
	   sq_hi_q <= sq_hi_d;
	   gt1_q   <= gt1_d;
	   gta_q   <= gta_d;
    END IF;
  END PROCESS reg_variables;
  --aritmeticke operace
  gt1_d   <= '1' WHEN (sq_hi_q - sq_lo_q) > 1 ELSE
             '0'; 
  sq_tmp  <= (sq_lo_q + sq_hi_q) srl 1;
  sq2_tmp <= sq_tmp * sq_tmp;
  gta_d   <= '1' WHEN sq2_tmp > a_q ELSE
             '0';
  --multiplexery pred registry
  sq_lo_d   <= (OTHERS => '0') WHEN we_lo = "01" ELSE
               sq_lo_q         WHEN we_lo = "00" ELSE
               sq_tmp;
  sq_hi_d   <= a_in            WHEN we_hi = "01" ELSE
               sq_hi_q         WHEN we_hi = "00" ELSE
               sq_tmp;
  a_d       <= a_in            WHEN we_hi = "01" ELSE
               a_q;
  gta       <= gta_q;
  gt1       <= gt1_q;
  sq_lo_out <= sq_lo_q;
END ARCHITECTURE rtl;