LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY tb IS
END ENTITY tb;

ARCHITECTURE behav OF tb IS

  CONSTANT clk_per : time    := 100 ns; --perioda hodin 
  CONSTANT n       : natural := 16;

  COMPONENT dp IS 
    GENERIC (
      n : natural := 16
    );
    PORT (
      clk        : IN  std_logic;
      res        : IN  std_logic;

      a_in       : IN  unsigned (n-1 downto 0);
      we_lo      : IN  std_logic_vector (1 DOWNTO 0);
      we_hi      : IN  std_logic_vector (1 DOWNTO 0);
      sq_lo_out  : OUT unsigned (n-1 downto 0);
      gt1        : OUT std_logic;
      gta        : OUT std_logic
 	 );
  END COMPONENT dp;

  SIGNAL clk_i       : std_logic;
  SIGNAL res_i       : std_logic;
  SIGNAL a_in_i      : unsigned (n-1 DOWNTO 0);
  SIGNAL we_lo_i     : std_logic_vector (1 DOWNTO 0);
  SIGNAL we_hi_i     : std_logic_vector (1 DOWNTO 0);
  SIGNAL gt1_i       : std_logic;
  SIGNAL gta_i       : std_logic;
  SIGNAL sq_lo_out_i : unsigned (n-1 DOWNTO 0);

BEGIN

  i_dp : dp 
    GENERIC MAP (
      n => n
    )
    PORT MAP (
      clk        => clk_i,
      res        => res_i,

      a_in       => a_in_i,
      we_lo      => we_lo_i,
      we_hi      => we_hi_i,
      sq_lo_out  => sq_lo_out_i,
      gt1        => gt1_i,
      gta        => gta_i
    );
  
  testbench_control : PROCESS

    PROCEDURE gen_clk (CONSTANT pulses : IN natural := 1) IS
    BEGIN
      FOR pulse_no IN 0 TO pulses - 1 LOOP
        clk_i <= '0';
        WAIT FOR clk_per / 2;
        clk_i <= '1';
        WAIT FOR clk_per / 2;
      END LOOP;
    END PROCEDURE gen_clk;

    FUNCTION square_root (a:natural) RETURN natural IS
      VARIABLE sq_hi:natural;
      VARIABLE sq_lo:natural;
    BEGIN
      sq_hi := a;
      sq_lo := 0;
      WHILE (sq_hi-sq_lo > 1) LOOP
        IF (((sq_hi+sq_lo)/2)**2 > a) THEN
          sq_hi := (sq_hi+sq_lo)/2;
        ELSE
          sq_lo := (sq_hi+sq_lo)/2;
        END IF;
      END LOOP;
      RETURN sq_lo;
    END FUNCTION square_root;

 
    PROCEDURE execute_one_cycle (CONSTANT a      : IN natural) IS
      VARIABLE a_vstup          : unsigned   (N-1   DOWNTO 0);
      --ocekavane vystupy
      VARIABLE odmocnina        : natural;
      VARIABLE odmocnina_ocek   : unsigned   (N-1 DOWNTO 0);
    BEGIN
      --konvertuj integer hodnoty na std_logic_vectory
      a_vstup        := to_unsigned  (a     , N  );
      odmocnina      := square_root  (a);
      odmocnina_ocek := to_unsigned(odmocnina,N);
      ASSERT FALSE
      REPORT natural'IMAGE(a) & " --> " & natural'IMAGE(odmocnina)
      SEVERITY NOTE;
      --prived hodnoty na vstup obvodu
      a_in_i <= a_vstup;
      --vygeneruj jeden hodinovy pulz - zapis do registru
      we_lo_i <= "01";
      we_hi_i <= "01";
      gen_clk;
		--dalsi hodinovy pulz pro aktualizaci gta a gt1 priznaku
      we_lo_i <= "00";
      we_hi_i <= "00";
      gen_clk;
      WHILE gt1_i = '1' LOOP
        --podle gta priznaku zapis registr
        IF gta_i = '1' THEN
          we_lo_i <= "00";
          we_hi_i <= "10";
        ELSE
          we_lo_i <= "10";
          we_hi_i <= "00";
        END IF;
        gen_clk;
		  --vypocti gta priznak
        we_lo_i <= "00"; 
        we_hi_i <= "00";
		  gen_clk;
      END LOOP;		
		
      --zkontroluj vystupni sbernice
      ASSERT odmocnina_ocek = sq_lo_out_i
      REPORT "tesbench.vhd: chybny vypocet odmocniny - na vystupu je " & 
              integer'IMAGE(to_integer(sq_lo_out_i)) & " misto " &
              integer'IMAGE(to_integer(odmocnina_ocek)) & "!"
      SEVERITY FAILURE;
      --konec
    END PROCEDURE execute_one_cycle;
 
  BEGIN
    --vyresetuj obvod a nastav vstupni signaly do def. hodnot
    res_i    <= '1';
    we_lo_i <= "00";
    we_hi_i <= "00";
    WAIT FOR 100 ns;
    res_i <= '0';
	 --nacti hodnoty

    FOR smycka_a IN 0 TO 255 LOOP
      execute_one_cycle(smycka_a*255);
    END LOOP;
    ASSERT FALSE
    REPORT "Simulace neodhalila zadne chyby"
    SEVERITY NOTE;
    WAIT;
  END PROCESS testbench_control;
  
END ARCHITECTURE behav;