library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.ALL;

ENTITY addr_gen IS 
  GENERIC (
    N : natural := 4
  );
  PORT (
    clk     : IN  std_logic;
    res     : IN  std_logic;
    gen_cmd : IN  std_logic_vector (  1 DOWNTO 0);

    lt14    : OUT std_logic;
    lt15    : OUT std_logic;
    addr    : OUT std_logic_vector (N-1 DOWNTO 0)
	);
END addr_gen;

ARCHITECTURE rtl OF addr_gen IS
  SIGNAL addr_q  : std_logic_vector (N-1 DOWNTO 0);
  SIGNAL addr_d  : std_logic_vector (N-1 DOWNTO 0);
  SIGNAL lt14_q  : std_logic;
  SIGNAL lt14_d  : std_logic;
  SIGNAL lt15_q  : std_logic;
  SIGNAL lt15_d  : std_logic;
BEGIN
  reg_address : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN
	   addr_q <= (OTHERS => '0');
		lt14_q <= '0';
		lt15_q <= '0';
	 ELSIF clk'EVENT AND clk='1' THEN
	   addr_q <= addr_d;
		lt14_q <= lt14_d;
		lt15_q <= lt15_d;
    END IF;
  END PROCESS reg_address;
  --aktualizace adresy
  addr_d <= (OTHERS => '0')    WHEN gen_cmd = "01" ELSE 
            std_logic_vector(unsigned(addr_q)+1) WHEN gen_cmd = "10" ELSE
				addr_q;
  addr   <= addr_q;
  --priznak
  lt14_d <= '1' WHEN unsigned(addr_q) < 14 ELSE
            '0';
  lt14   <= lt14_q;
  lt15_d <= '1' WHEN unsigned(addr_q) < 15 ELSE
            '0';
  lt15   <= lt15_q;
END ARCHITECTURE rtl;
