library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.ALL;

ENTITY dp IS 
  GENERIC (
    N : natural := 16
  );
  PORT (
    clk     : IN  std_logic;
    res     : IN  std_logic;

    we_acc  : IN  std_logic_vector ( 1  DOWNTO 0);
    we_mem  : IN  std_logic;
    addr    : IN  std_logic_vector ( 3  DOWNTO 0);
    data_in : IN  signed           (N-1 DOWNTO 0);

    acc_out : OUT signed           (N-1 DOWNTO 0)
	);
END dp;

ARCHITECTURE rtl OF dp IS

  SIGNAL rd_data : std_logic_vector (N-1 DOWNTO 0);
  SIGNAL acc_q   : signed           (N-1 DOWNTO 0);
  SIGNAL acc_d   : signed           (N-1 DOWNTO 0);  
 
  COMPONENT ram IS 
    GENERIC (
      words : natural := 16;
	   no_b  : natural :=  4;
	   bits  : natural := 16
    );
    PORT (
      clk      : IN  std_logic;
	   wr       : IN  std_logic;
  	   rd_addr  : IN  std_logic_vector (no_b-1 DOWNTO 0);
	   wr_addr  : IN  std_logic_vector (no_b-1 DOWNTO 0);
      data_in  : IN  std_logic_vector (bits-1 DOWNTO 0);
	   data_out : OUT std_logic_vector (bits-1 DOWNTO 0)
    );
  END COMPONENT ram;
  
BEGIN

  ram16x16 : ram 
    GENERIC MAP (
      words => 16,
		no_b  =>  4,
	   bits  => N
    )
    PORT MAP (
      clk      => clk,
	   wr       => we_mem,
  	   rd_addr  => addr,
	   wr_addr  => addr,
      data_in  => std_logic_vector(data_in),
	   data_out => rd_data
    );

  reg_variables : PROCESS (clk, res)
  BEGIN
    IF res = '1' THEN
	   acc_q   <= (OTHERS => '0');
	 ELSIF clk'EVENT AND clk='1' THEN
	   acc_q   <= acc_d;
    END IF;
  END PROCESS reg_variables;
  
  --scitani vzorku
  acc_d   <= (OTHERS => '0')         WHEN we_acc = "01" ELSE
             acc_q + signed(rd_data) WHEN we_acc = "10" ELSE
				 acc_q;
  --deleno 16 a znamenkove rozsireni			 
  acc_out <= acc_q(15)&acc_q(15)&acc_q(15)&acc_q(15)&acc_q(15 DOWNTO 4);
END ARCHITECTURE rtl;