LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY tb IS
END ENTITY tb;

ARCHITECTURE behav OF tb IS

  CONSTANT clk_per : time    := 100 ns; --perioda hodin 
  CONSTANT N       : natural := 16;

  COMPONENT addr_gen IS 
    GENERIC (
      N : natural := 4
    );
    PORT (
      clk     : IN  std_logic;
      res     : IN  std_logic;
      gen_cmd : IN  std_logic_vector (  1 DOWNTO 0);
      lt14    : OUT std_logic;
      lt15    : OUT std_logic;
      addr    : OUT std_logic_vector (N-1 DOWNTO 0)
	  );
  END COMPONENT addr_gen;

  COMPONENT dp IS 
    GENERIC (
      N : natural := 16
    );
    PORT (
      clk     : IN  std_logic;
      res     : IN  std_logic;

      we_acc  : IN  std_logic_vector (  1 DOWNTO 0);
      we_mem  : IN  std_logic;
      addr    : IN  std_logic_vector (  3 DOWNTO 0);
      data_in : IN  signed           (N-1 DOWNTO 0);

      acc_out : OUT signed           (N-1 DOWNTO 0)
	  );
  END COMPONENT dp;

  SIGNAL clk_i     : std_logic;
  SIGNAL res_i     : std_logic;
  SIGNAL gen_cmd_i : std_logic_vector (  1 DOWNTO 0);
  SIGNAL we_acc_i  : std_logic_vector (  1 DOWNTO 0);
  SIGNAL we_mem_i  : std_logic;
  SIGNAL addr_i    : std_logic_vector (  3 DOWNTO 0);
  SIGNAL lt14_i    : std_logic;
  SIGNAL lt15_i    : std_logic;
  
  SIGNAL data_in_i : signed           (N-1 DOWNTO 0);
  SIGNAL acc_out_i : signed           (N-1 DOWNTO 0);
BEGIN

  i_dp : dp 
    GENERIC MAP (
      N => N
    )
    PORT MAP (
      clk    => clk_i,
      res    => res_i,

      we_acc  => we_acc_i,
      we_mem  => we_mem_i,
      addr    => addr_i,
      data_in => data_in_i,

      acc_out => acc_out_i
    );
	 
  i_addr_gen : addr_gen 
    GENERIC MAP (
      N => 4
    )
    PORT MAP (
      clk     => clk_i,
      res     => res_i,
		lt14    => lt14_i,
		lt15    => lt15_i,
      gen_cmd => gen_cmd_i,
      addr    => addr_i
	  );
  
  testbench_control : PROCESS

    PROCEDURE gen_clk (CONSTANT pulses : IN natural := 1) IS
    BEGIN
      FOR pulse_no IN 0 TO pulses - 1 LOOP
        clk_i <= '0';
        WAIT FOR clk_per / 2;
        clk_i <= '1';
        WAIT FOR clk_per / 2;
      END LOOP;
    END PROCEDURE gen_clk;
    
    TYPE tarr IS ARRAY (0 TO 15) OF integer;
    
    FUNCTION average (val : tarr) RETURN integer IS
      VARIABLE acc:integer;
    BEGIN
      acc := 0;
      FOR index IN 0 TO 15 LOOP
        acc := acc + val(index);
      END LOOP;
      RETURN acc/16;
    END FUNCTION average;

    PROCEDURE execute_one_cycle (CONSTANT a      : IN tarr) IS
      VARIABLE a_vstup       : unsigned   (N-1 DOWNTO 0);
      --ocekavane vystupy
      VARIABLE prumer        : integer;
      VARIABLE prumer_ocek   : signed     (N-1 DOWNTO 0);
    BEGIN
      --konvertuj integer hodnoty na std_logic_vectory
      prumer       := average(a);
      prumer_ocek  := to_signed(prumer,N);
      ASSERT FALSE
      REPORT " PRUMER --> " & integer'IMAGE(prumer)
      SEVERITY NOTE;
		
		--incializace adresniho generatoru
		gen_cmd_i <= "01";
		we_acc_i  <= "00";
		we_mem_i  <= '0';
		gen_clk;
		--zapis hodnoty do pameti
		WHILE true LOOP
		  we_acc_i  <= "01"; --reset akumulatoru
		  we_mem_i  <= '1';
		  IF lt14_i = '1' THEN
		    gen_cmd_i <= "10"; --inkrement adresy
		  ELSE
		    gen_cmd_i <= "01"; --reset adresy, opet od nuly
		  END IF;
		  data_in_i <= to_signed(a(to_integer(unsigned(addr_i))),N);
		  gen_clk;
		  IF lt15_i = '0' THEN
		    EXIT;
		  END IF;
		END LOOP;
      we_acc_i  <= "01"; --reset akumulatoru
      gen_cmd_i <= "10"; --inkrement adresy
		we_mem_i  <= '0';
		gen_clk; --pamet musi vybavit adresu
		--vypocet prumeru
		WHILE true LOOP
		  we_acc_i  <= "10"; --akumulator je povolen
		  we_mem_i  <= '0';
 	     gen_cmd_i <= "10"; --inkrement adresy
		  gen_clk;
		  IF lt15_i = '0' THEN
		    EXIT;
		  END IF;
		END LOOP;
      gen_cmd_i <= "01"; --reset adresy, opet od nuly		
      we_acc_i  <= "10"; --akumulator je povolen
		gen_clk; --posledni hodnota v pipeline
      --zkontroluj vystupni sbernice
      ASSERT prumer_ocek = acc_out_i
      REPORT "tesbench.vhd: chybny vypocet prumeru - na vystupu je " & 
              integer'IMAGE(to_integer(acc_out_i)) & " misto " &
              integer'IMAGE(to_integer(prumer_ocek)) & "!"
      SEVERITY FAILURE;
      --konec
    END PROCEDURE execute_one_cycle;
 
 
     VARIABLE data : tarr;
  BEGIN
    --vyresetuj obvod a nastav vstupni signaly do def. hodnot
    res_i     <= '1';
    gen_cmd_i <= "00";
	 we_acc_i  <= "00";
	 we_mem_i  <= '0';
    WAIT FOR 100 ns;
    res_i <= '0';
	 --vypocet
    FOR smycka_a IN 0 TO 255 LOOP
	   FOR index IN 0 TO 15 LOOP
		  data(index) := smycka_a + index;
		END LOOP;
      execute_one_cycle(data);
    END LOOP;
	 
    ASSERT FALSE
    REPORT "Simulace neodhalila zadne chyby"
    SEVERITY NOTE;
    WAIT;
  END PROCESS testbench_control;
  
END ARCHITECTURE behav;