LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY dual_port_sync_ram IS
  PORT (
    clk1  : IN  std_logic;
	  we1   : IN  std_logic;
	  addr1 : IN  std_logic_vector ( 7 DOWNTO 0);
	  di1   : IN  std_logic_vector (15 DOWNTO 0);
	  do1   : OUT std_logic_vector (15 DOWNTO 0);
	 
    clk2  : IN  std_logic;
	  addr2 : IN  std_logic_vector ( 7 DOWNTO 0);
	  do2   : OUT std_logic_vector (15 DOWNTO 0)
  );
END ENTITY dual_port_sync_ram;

ARCHITECTURE rtl OF dual_port_sync_ram IS
  TYPE   tram_array IS ARRAY (255 DOWNTO 0) OF std_logic_vector (15 DOWNTO 0);
  SIGNAL ram : tram_array;
  SIGNAL read_addr1 : std_logic_vector (7 DOWNTO 0);
  SIGNAL read_addr2 : std_logic_vector (7 DOWNTO 0);
BEGIN

  sync_ram_impl1:PROCESS (clk1)
  BEGIN
    IF clk1'EVENT AND clk1 = '1' THEN
      IF we1 = '1' THEN
        ram(to_integer(unsigned(addr1))) <= di1;
      END IF;
      read_addr1 <= addr1;
    END IF;
  END PROCESS sync_ram_impl1;
  do1 <= ram(to_integer(unsigned(read_addr1)));

  sync_ram_impl2:PROCESS (clk2)
  BEGIN
    IF clk2'EVENT AND clk2 = '1' THEN
      read_addr2 <= addr2;
    END IF;
  END PROCESS sync_ram_impl2;
  do2 <= ram(to_integer(unsigned(read_addr2)));

END ARCHITECTURE rtl;